import re
import threading
import requests


def validate_currency(currency_str):
    return len(currency_str) == 3


def validate_amount(amount_str):
    try:
        return float(amount_str)
    except ValueError:
        return False


def validate_phone(phone_str):
    try:
        return re.match("^\+\d{1,3}\d{3,}$", phone_str)
    except ValueError:
        return False


def validate_data_units(data_unit):
    if data_unit in ["MB", "GB"]:
        return True
    return False


def validate_data_validity(data_validity):
    if data_validity in ["Day", "Week", "Month"]:
        return True
    return False


def validate_keys(test_dict, valid_keys_set):
    if set(test_dict.keys()) == valid_keys_set:
        return True
    return False


class AfricasTalkingException(Exception):
    pass


class Service(object):
    def __init__(self, username, api_key):
        if not isinstance(username, str):
            raise RuntimeError("username has to be of type str.")

        if not isinstance(api_key, str):
            raise RuntimeError("api_key has to be of type str.")

        self._PRODUCTION_DOMAIN = "africastalking.com"
        self._SANDBOX_DOMAIN = "sandbox.africastalking.com"

        self._username = username
        self._api_key = api_key
        self._headers = {
            "Accept": "application/json",
            "User-Agent": "africastalking-python/2.0.0",
            "apiKey": self._api_key,
        }
        self._baseUrl = "https://api." + self._PRODUCTION_DOMAIN

        self._contentUrl = "https://content." + self._PRODUCTION_DOMAIN

        self._init_service()

    def _is_sandbox(self):
        return self._username == "sandbox"

    def _make_url(self, path, content=None):
        if content is None:
            return self._baseUrl + path
        else:
            return self._contentUrl + path

    def _init_service(self):
        raise NotImplementedError

    @staticmethod
    def __make_get_request(url, headers, data, params, callback=None):
        res = requests.get(url=url, headers=headers, params=params, data=data)

        if callback is None or callback == {}:
            return res
        else:
            callback(res)

    @staticmethod
    def __make_post_request(url, headers, data, params, callback=None):
        res = requests.post(
            url=url,
            headers=headers,
            params=params,
            data=data,
        )
        if callback is None or callback == {}:
            return res
        else:
            callback(res)

    def _make_request(self, url, method, headers, data, params, callback=None):
        method = method.upper()
        if callback is None:
            if method == "GET":
                res = self.__make_get_request(
                    url=url,
                    headers=headers,
                    data=data,
                    params=params,
                )
            elif method == "POST":
                res = self.__make_post_request(
                    url=url,
                    headers=headers,
                    data=data,
                    params=params,
                )
            else:
                raise AfricasTalkingException("Unexpected HTTP method: " + method)

            if 200 <= res.status_code < 300:
                if res.headers.get("content-type") == "application/json":
                    return res.json()
                else:
                    return res.text
            else:
                raise AfricasTalkingException(res.text)
        elif not callable(callback):
            raise RuntimeError("callback has to be callable. e.g. a function")
        else:

            def cb(response):
                if 200 <= response.status_code < 300:
                    if response.headers.get("content-type") == "application/json":
                        callback(None, response.json())
                    else:
                        callback(None, response.text)
                else:
                    callback(AfricasTalkingException(response.text), None)

            if method == "GET":
                _target = self.__make_get_request
            elif method == "POST":
                _target = self.__make_post_request
            else:
                raise AfricasTalkingException("Unexpected HTTP method: " + method)

            thread = threading.Thread(
                target=_target, args=(url, headers, data, params, cb)
            )
            thread.start()
            return thread


class APIService(Service):
    def __init__(self, username, api_key):
        super(APIService, self).__init__(username, api_key)

    def _init_service(self):
        self._baseUrl = "https://api."
        if self._is_sandbox():
            self._baseUrl += self._SANDBOX_DOMAIN
            self._contentUrl = self._baseUrl
        else:
            self._baseUrl += self._PRODUCTION_DOMAIN
            self._contentUrl = "https://content." + self._PRODUCTION_DOMAIN



from .Token import TokenService
from .Application import ApplicationService
from .Airtime import AirtimeService
from .SMS import SMSService
from .Voice import VoiceService
from .MobileData import MobileDataService
from .Insights import InsightService

SMS = None
Airtime = None
USSD = None
Voice = None
Application = None
Token = None
MobileData = None
Insights = None


def initialize(username, api_key):
    if username is None or api_key is None:
        raise RuntimeError("Invalid username and/or api_key")

    globals()["SMS"] = SMSService(username, api_key)
    globals()["Airtime"] = AirtimeService(username, api_key)
    globals()["Voice"] = VoiceService(username, api_key)
    globals()["Application"] = ApplicationService(username, api_key)
    globals()["Token"] = TokenService(username, api_key)
    globals()["MobileData"] = MobileDataService(username, api_key)
    globals()["Insights"] = InsightService(username, api_key)
    # globals()['USSD'] = USSDService(username, api_key)



from .Service import Service, validate_phone


class VoiceService(Service):
    def __init__(self, username, api_key):
        super(VoiceService, self).__init__(username, api_key)

    def _init_service(self):
        self._baseUrl = "https://voice."
        if self._is_sandbox():
            self._baseUrl += self._SANDBOX_DOMAIN
        else:
            self._baseUrl += self._PRODUCTION_DOMAIN

    def call(self, callFrom, callTo, callback=None):
        for phone_number in callTo:
            if not validate_phone(phone_number):
                raise ValueError("Invalid callTo phone number:" + phone_number)

        callTo = ",".join(callTo)
        url = self._make_url("/call")
        data = {
            "username": self._username,
            "from": callFrom,
            "to": callTo,
        }
        headers = dict(self._headers)
        headers["Content-Type"] = "application/x-www-form-urlencoded"
        return self._make_request(
            url,
            "POST",
            headers,
            params=None,
            data=data,
            callback=callback,
        )

    def fetch_queued_calls(self, phone_number, callback=None):
        if not validate_phone(phone_number):
            raise ValueError("Invalid phone number")

        url = self._make_url("/queueStatus")
        data = {
            "username": self._username,
            "phoneNumbers": phone_number,
        }
        return self._make_request(
            url,
            "POST",
            headers=self._headers,
            params=None,
            data=data,
            callback=callback,
        )

    def media_upload(self, phone_number, url, callback=None):
        if not validate_phone(phone_number):
            raise ValueError("Invalid phone number")

        call_url = self._make_url("/mediaUpload")
        data = {
            "username": self._username,
            "phoneNumber": phone_number,
            "url": url,
        }
        return self._make_request(
            call_url,
            "POST",
            headers=self._headers,
            params=None,
            data=data,
            callback=callback,
        )



"""
Insights

fetchApplicationData(): Fetch app info i.e. balance.
"""

import africastalking
import unittest
from test import USERNAME, API_KEY

africastalking.initialize(USERNAME, API_KEY)
service = africastalking.Insights


class TestInsightsService(unittest.TestCase):
    def test_sim_swap_state(self):
        res = service.check_sim_swap_state(["+254712345678"])
        assert res["status"] == "Processed"


if __name__ == "__main__":
    unittest.main()



import json
from schema import Schema, And, Optional
from .Service import (
    Service,
    validate_phone,
    validate_data_units,
    validate_data_validity,
)


class MobileDataService(Service):
    def __init__(self, username, api_key):
        super(MobileDataService, self).__init__(username, api_key)

    def _init_service(self):
        self._baseUrl = "https://bundles."
        if self._is_sandbox():
            self._baseUrl += self._SANDBOX_DOMAIN
        else:
            self._baseUrl += self._PRODUCTION_DOMAIN

    def send(self, product_name, recipients, callback=None):
        schema = Schema(
            [
                {
                    "phoneNumber": And(str, lambda s: validate_phone(s)),
                    "quantity": And(lambda f: float(f) > 0),
                    "unit": And(str, lambda s: validate_data_units(s)),
                    "validity": And(str, lambda s: validate_data_validity(s)),
                    Optional("metadata"): And(dict),
                }
            ]
        )
        recipients = schema.validate(recipients)
        url = self._make_url("/mobile/data/request")
        headers = dict(self._headers)
        headers["Content-Type"] = "application/json"
        data = {
            "username": self._username,
            "productName": product_name,
            "recipients": recipients,
        }
        data = json.dumps(data)
        return self._make_request(
            url, "POST", headers=headers, params=None, data=data, callback=callback
        )

    def find_transaction(self, transaction_id, callback=None):
        url = self._make_url("/query/transaction/find")
        headers = dict(self._headers)
        headers["Content-Type"] = "application/json"
        params = {
            "username": self._username,
            "transactionId": transaction_id,
        }
        return self._make_request(
            url, "GET", headers=headers, data=None, params=params, callback=callback
        )

    def fetch_wallet_balance(self, callback=None):
        url = self._make_url("/query/wallet/balance")
        params = {
            "username": self._username,
        }
        return self._make_request(
            url,
            "GET",
            data=None,
            headers=self._headers,
            params=params,
            callback=callback,
        )



import json

from .Service import APIService


class TokenService(APIService):
    def __init__(self, username, api_key):
        super(TokenService, self).__init__(username, api_key)

    def generate_auth_token(self, callback=None):
        url = self._make_url("/auth-token/generate")
        headers = dict(self._headers)
        headers["Content-Type"] = "application/json"
        data = json.dumps({"username": self._username})

        return self._make_request(
            url, "POST", headers, params=None, data=data, callback=callback
        )

    def create_checkout_token(self, phone_number, callback=None):
        url = self._make_url("/checkout/token/create")
        headers = dict(self._headers)
        headers["Content-Type"] = "application/x-www-form-urlencoded"
        data = {"phoneNumber": phone_number}

        return self._make_request(
            url, "POST", headers, params=None, data=data, callback=callback
        )



import json
from .Service import Service, validate_phone


class InsightService(Service):
    def __init__(self, username, api_key):
        super(InsightService, self).__init__(username, api_key)

    def _init_service(self):
        self._baseUrl = "https://insights."
        if self._is_sandbox():
            self._baseUrl += self._SANDBOX_DOMAIN + "/v1"
        else:
            self._baseUrl += self._PRODUCTION_DOMAIN + "/v1"

    def check_sim_swap_state(self, phone_numbers, callback=None):
        url = self._make_url("/sim-swap")
        headers = dict(self._headers)
        headers["Content-Type"] = "application/json"
        for phone_number in phone_numbers:
            if not validate_phone(phone_number):
                raise ValueError(
                    "Invalid phone number in list of phone numbers: %s", phone_number
                )
        data = {
            "username": self._username,
            "phoneNumbers": phone_numbers,
        }
        data = json.dumps(data)
        return self._make_request(
            url, "POST", headers=headers, params=None, data=data, callback=callback
        )



"""
SMS

send(message: String, recipients: List<String>, senderId: Optional<String>, enqueue: Optional<Boolean>): Send a bulk message to recipients, optionally from senderId (Short Code or Alphanumeric).

sendPremium(message: String, keyword: String, linkId: String, recipients: List<String>, senderId: Optional<String>, retryDurationInHours: Optional<Integer>): Send a premium SMS

fetchMessages(lastReceivedId: Optional<Integer>): Fetch your messages

fetchSubscriptions(shortCode: String, keyword: String, lastReceivedId: Optional<Integer>): Fetch your premium subscription data

createSubscription(shortCode: String, keyword: String, phoneNumber: String): Create a premium subscription
"""

import africastalking
import unittest
from test import USERNAME, API_KEY

africastalking.initialize(USERNAME, API_KEY)
token_service = africastalking.Token
service = africastalking.SMS


class TestSmsService(unittest.TestCase):
    def test_send(self):
        res = service.send(
            "test_send()",
            ["+254718769882", "+254718769881"],
            enqueue=True,
            sender_id="AT2FA",
        )
        recipients = res["SMSMessageData"]["Recipients"]
        assert len(recipients) == 2
        assert recipients[0]["status"] == "Success"

    def test_heavy_single_send(self):
        count = 1000
        phone_numbers = list(
            map(lambda x: str("+254718" + str(x + count)), range(1, count))
        )

        def on_finish(error, data):
            if error:
                raise error
            recipients = data["SMSMessageData"]["Recipients"]
            assert len(recipients) <= count

        service.send(
            "test_heavy_single_send()",
            phone_numbers,
            enqueue=True,
            sender_id="AT2FA",
            callback=on_finish,
        )

    def test_send_premium(self):
        res = service.send_premium(
            "test_send_premium()",
            "AT2FA",
            ["+254718769882", "+254718769881"],
            "KiKi",
            "Linky",
            retry_duration_in_hours=10,
        )
        recipients = res["SMSMessageData"]["Recipients"]
        assert len(recipients) == 2
        assert recipients[0]["status"] == "Success"

    def test_fetch_messages(self):
        res = service.fetch_messages(0)
        assert len(res) >= 0

    def test_fetch_subscriptions(self):
        res = service.fetch_subscriptions(
            short_code=78942, keyword="KiKi", last_received_id=0
        )
        assert len(res) >= 0

    def test_create_subscription(self):
        checkout_token = token_service.create_checkout_token("+254718769882")
        res = service.create_subscription(
            short_code=78942,
            keyword="KiKi",
            phone_number="+254718769882",
            checkout_token=checkout_token["token"],
        )
        assert res["description"] == "Waiting for user input"

    def test_delete_subscription(self):
        res = service.delete_subscription(
            short_code=78942, keyword="KiKi", phone_number="+254718769882"
        )
        assert res["status"] == "Success"


if __name__ == "__main__":
    unittest.main()



import json
from .Service import (
    APIService,
    validate_amount,
    validate_phone,
    validate_currency,
    validate_keys,
)


class AirtimeService(APIService):
    def __init__(self, username, api_key):
        super(AirtimeService, self).__init__(username, api_key)

    def _init_service(self):
        super(AirtimeService, self)._init_service()
        self._baseUrl = self._baseUrl + "/version1/airtime"

    def send(
        self,
        phone_number=None,
        amount=None,
        currency_code=None,
        recipients=None,
        idempotency_key=None,
        callback=None,
        max_num_retry=None,
    ):
        def join_amount_and_currency(obj):
            obj["amount"] = " ".join([str(obj["currency_code"]), str(obj["amount"])])
            del obj["currency_code"]
            return obj

        def value_validator(phoneNumber, amount, currency_code):
            if not validate_phone(phoneNumber):
                return "Invalid phone number"
            elif not validate_amount(amount):
                return "Invalid amount"
            elif not validate_currency(currency_code):
                return "Invalid currency code"
            else:
                return False

        if recipients is None:
            if not all([phone_number, amount, currency_code]):
                raise (
                    ValueError(
                        "must specify phoneNumber, currencyCode and amount for recipient"
                    )
                )

            recipients = [
                {
                    "phoneNumber": str(phone_number),
                    "amount": str(amount),
                    "currency_code": str(currency_code),
                }
            ]

        for recipient in recipients:
            if not validate_keys(recipient, {"phoneNumber", "amount", "currency_code"}):
                raise (
                    ValueError(
                        "must specify phoneNumber, currencyCode and amount for recipient: %s"
                        % (recipient)
                    )
                )

            else:
                phoneNumber = recipient["phoneNumber"]
                amount = recipient["amount"]
                currency_code = recipient["currency_code"]
                validation_err = value_validator(phoneNumber, amount, currency_code)

                if validation_err:
                    raise (
                        ValueError(
                            "Recipient data error: %s in %s"
                            % (validation_err, recipient)
                        )
                    )
                else:
                    join_amount_and_currency(recipient)

        url = self._make_url("/send")
        data = {"username": self._username, "recipients": json.dumps(recipients)}
        if idempotency_key:
            self._headers["Idempotency-Key"] = idempotency_key
        if max_num_retry:
            data["maxNumRetry"] = max_num_retry
        return self._make_request(
            url,
            "POST",
            headers=self._headers,
            params=None,
            data=data,
            callback=callback,
        )



#!/usr/bin/env python
from setuptools import setup
import sys
import os

version = "1.2.9"

long_description = open("README.md").read()

if sys.argv[-1] == "publish":
    os.system("python setup.py sdist upload")
    sys.exit()

setup(
    name="africastalking",
    version=version,
    packages=["africastalking"],
    description="Official Africa's Talking Python SDK",
    data_files=[("", ["README.md"])],
    license="MIT",
    author="Africa's Talking",
    install_requires=["requests>=v2.18.4", "schema>=0.6.7"],
    python_requires=">=2.7.10",
    author_email="info@africastalking.com",
    url="https://github.com/AfricasTalkingLtd/africastalking-python",
    download_url="https://codeload.github.com/AfricasTalkingLtd/africastalking-python/tar.gz/"
    + version,
    keywords="ussd voice sms mpesa card bank b2b b2c sender_id payments airtime africastalking",
    classifiers=[],
    long_description=long_description,
    long_description_content_type="text/markdown",
)



from .Service import APIService


class ApplicationService(APIService):
    def __init__(self, username, api_key):
        super(ApplicationService, self).__init__(username, api_key)

    def _init_service(self):
        super(ApplicationService, self)._init_service()
        self._baseUrl = self._baseUrl + "/version1"

    def fetch_application_data(self, callback=None):
        url = self._make_url("/user")
        params = {"username": self._username}
        return self._make_request(
            url,
            "GET",
            headers=self._headers,
            params=params,
            data=None,
            callback=callback,
        )



import africastalking
import unittest
from test import USERNAME, API_KEY

africastalking.initialize(USERNAME, API_KEY)
service = africastalking.MobileData


class TestMobileDataService(unittest.TestCase):
    def test_mobile_data(self):
        try:
            recipient = {
                "phoneNumber": "+254711223344",
                "quantity": 10,
                "unit": "GB",
                "validity": "Month",
                "metadata": {"some": "information"},
            }
            res = service.send(product_name="TestProduct", recipients=[recipient])
            assert res["status"] is not None
        except africastalking.Service.AfricasTalkingException:
            assert True



from .Service import APIService, validate_phone


class SMSService(APIService):
    def __init__(self, username, api_key):
        super(SMSService, self).__init__(username, api_key)

    def _init_service(self):
        super(SMSService, self)._init_service()
        self._baseUrl = self._baseUrl + "/version1"
        if self._contentUrl:
            self._contentUrl = self._contentUrl + "/version1"

    def send(self, message, recipients, sender_id=None, enqueue=False, callback=None):
        for phone in recipients:
            if not validate_phone(phone):
                raise ValueError("Invalid phone number: " + phone)

        url = self._make_url("/messaging")
        data = {
            "username": self._username,
            "to": ",".join(recipients),
            "message": message,
            "bulkSMSMode": 1,
        }

        if sender_id is not None:
            data["from"] = sender_id

        if enqueue:
            data["enqueue"] = 1

        return self._make_request(
            url,
            "POST",
            headers=self._headers,
            params=None,
            data=data,
            callback=callback,
        )

    def send_premium(
        self,
        message,
        short_code,
        recipients,
        keyword=None,
        link_id=None,
        retry_duration_in_hours=None,
        callback=None,
    ):
        for phone in recipients:
            if not validate_phone(phone):
                raise ValueError("Invalid phone number: " + phone)

        url = self._make_url("/messaging", content=True)
        data = {
            "username": self._username,
            "to": ",".join(recipients),
            "from": short_code,
            "message": message,
            "bulkSMSMode": 0,
        }

        if link_id is not None:
            data["linkId"] = link_id

        if keyword is not None:
            data["keyword"] = keyword

        if retry_duration_in_hours is not None:
            data["retryDurationInHours"] = retry_duration_in_hours

        return self._make_request(
            url,
            "POST",
            headers=self._headers,
            params=None,
            data=data,
            callback=callback,
        )

    def fetch_messages(self, last_received_id=None, callback=None):
        url = self._make_url("/messaging")
        params = {"username": self._username}

        if last_received_id is not None:
            params["lastReceivedId"] = last_received_id

        return self._make_request(
            url,
            "GET",
            headers=self._headers,
            params=params,
            data=None,
            callback=callback,
        )

    def fetch_subscriptions(
        self, short_code, keyword, last_received_id=None, callback=None
    ):
        url = self._make_url("/subscription", content=True)
        params = {
            "username": self._username,
            "shortCode": short_code,
            "keyword": keyword,
        }

        if last_received_id is not None:
            params["lastReceivedId"] = last_received_id

        return self._make_request(
            url,
            "GET",
            headers=self._headers,
            params=params,
            data=None,
            callback=callback,
        )

    def create_subscription(
        self, short_code, keyword, phone_number, checkout_token, callback=None
    ):
        if not validate_phone(phone_number):
            raise ValueError("Invalid phone number")

        url = self._make_url("/subscription/create", content=True)
        data = {
            "username": self._username,
            "shortCode": short_code,
            "keyword": keyword,
            "phoneNumber": phone_number,
            "checkoutToken": checkout_token,
        }

        return self._make_request(
            url,
            "POST",
            headers=self._headers,
            data=data,
            params=None,
            callback=callback,
        )

    def delete_subscription(self, short_code, keyword, phone_number, callback=None):
        if not validate_phone(phone_number):
            raise ValueError("Invalid phone number")

        url = self._make_url("/subscription/delete", content=True)
        data = {
            "username": self._username,
            "shortCode": short_code,
            "keyword": keyword,
            "phoneNumber": phone_number,
        }

        return self._make_request(
            url,
            "POST",
            headers=self._headers,
            data=data,
            params=None,
            callback=callback,
        )

    def send_hashed(
        self, message, masked_number, sender_id, telco="Safaricom", callback=None
    ):
        url = self._make_url("/messaging/bulk")
        data = {
            "username": self._username,
            "maskedNumber": masked_number,
            "telco": telco,
            "message": message,
            "senderId": sender_id,
            "phoneNumbers": [],
        }

        return self._make_request(
            url,
            "POST",
            headers=self._headers,
            params=None,
            data=data,
            callback=callback,
        )
